(ns reqrep.Client
  (:use clojure.pprint
        reqrep.Utils
        reqrep.Api)
  (:require [reqrep.zhelpers :as mq]))

(defrecord Client [params]
  Runnable
  (run [this]
    (let [{:keys [runs endpoint func args]} params
          num (Integer/parseInt runs)
          ctx (mq/context numberOfCpus)
          sock (mq/socket ctx mq/req)]
      (doto sock
          (.setSendTimeOut 1000)
          (.setReceiveTimeOut 1000))
      (dotimes [i numberOfCpus]
        (let [tcpConn (format "%s%d" endpoint i)]
          (println (format "\nConnecting to: %s" tcpConn))
          (mq/connect sock tcpConn)))
      (dotimes [i num]
        (if (= 0 i)
          (do (mq/send sock
                       (writeMsg (make-api-data {:func api-service-description})))
              (let [service-description (readMsg (mq/recv-str sock))]
                (println "\nRemote Service Info: ")
                (print-table [:name :arglists :doc] (-> service-description :data))
                )))
        (mq/send sock (writeMsg (make-api-data {:index (+ 1 i)
                                                :total num
                                                :func func
                                                :data args})))
        (try
          (let [contents (readMsg (mq/recv-str sock))
                {:keys [func data status]} contents]
            (println (format "\n%s: [%s => RemoteApi><%s => %s]" status args func data)))
          (catch Exception e (doall
                              (println "Terminating client....")
                              (. sock close)
                              (RuntimeException. (.getMessage e))
                              ))))
      (. sock close)
      (. ctx term)
      )))

(def endpoint "tcp://127.0.0.1:4444")

(defn -main [numberOfRuns func args]
  ;; Start Client
  (-> {:runs numberOfRuns
       :endpoint (format "%s" endpoint)
       :func func
       :args args} ->Client Thread. .start))
