(ns reqrep.Server
  (:use clojure.pprint
        reqrep.Utils
        reqrep.Api
        reqrep.Service)
  (:require [reqrep.zhelpers :as mq]))

(defrecord Worker [params]
  Runnable
  (run [this]
    (let [{:keys [name endpoint]} params
          ctx (mq/context numberOfCpus)
          sock (mq/socket ctx mq/rep)]
    (println (format "\nServer ready at: %s" endpoint))
    (mq/bind sock endpoint)
    (loop []
      (let [;; Read message contents
            contents (readMsg (mq/recv-str sock))
            {:keys [func data index total]} contents
            funct (grab-api-fn func api-namespace)
            args (. contents toClj)
            describeService (fn []
                              (let [description (make-api-data {:func api-service-description
                                                                :data (api-info api-namespace)
                                                                :status (format "Result from %s @ %s"
                                                                                      name endpoint)})]
                                (println "\nService Description: ")
                                (print-table [:name :arglists :doc] (-> description :data))
                                (mq/send sock (writeMsg description))))
            executeServiceMethod (fn []
                                   (println (format "\n%s received message #%d out of %d with contents:"
                                                    endpoint index total))
                                   (pprint contents)
                                   (println (format "Invoking %s on %s with arguments %s" func name args))
                                   (mq/send sock
                                            (writeMsg (make-api-data {:func func
                                                                      :data (funct args)
                                                                      :status (format "Result from %s @ %s"
                                                                                      name endpoint)}))))]
        (if (= api-service-description func)
          (describeService)
          (executeServiceMethod))
        (recur)
        ))
    )))

(def endpoint "tcp://127.0.0.1:4444")

(defn -main []
  (let [numberOfCpus (.availableProcessors (Runtime/getRuntime))]

    ;; Start Workers
    (dotimes [i numberOfCpus] (-> {:name (format "worker%d" i)
                                   :endpoint (format "%s%d"  endpoint i)} ->Worker Thread. .start))
    ))
